/****************************************************************************************
*
*   Disclaimer   This software code and all associated documentation, comments or other
*  of Warranty:  information (collectively "Software") is provided "AS IS" without
*                warranty of any kind. MICRON TECHNOLOGY, INC. ("MTI") EXPRESSLY
*                DISCLAIMS ALL WARRANTIES EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
*                TO, NONINFRINGEMENT OF THIRD PARTY RIGHTS, AND ANY IMPLIED WARRANTIES
*                OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE. MTI DOES NOT
*                WARRANT THAT THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE
*                OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE.
*                FURTHERMORE, MTI DOES NOT MAKE ANY REPRESENTATIONS REGARDING THE USE OR
*                THE RESULTS OF THE USE OF THE SOFTWARE IN TERMS OF ITS CORRECTNESS,
*                ACCURACY, RELIABILITY, OR OTHERWISE. THE ENTIRE RISK ARISING OUT OF USE
*                OR PERFORMANCE OF THE SOFTWARE REMAINS WITH YOU. IN NO EVENT SHALL MTI,
*                ITS AFFILIATED COMPANIES OR THEIR SUPPLIERS BE LIABLE FOR ANY DIRECT,
*                INDIRECT, CONSEQUENTIAL, INCIDENTAL, OR SPECIAL DAMAGES (INCLUDING,
*                WITHOUT LIMITATION, DAMAGES FOR LOSS OF PROFITS, BUSINESS INTERRUPTION,
*                OR LOSS OF INFORMATION) ARISING OUT OF YOUR USE OF OR INABILITY TO USE
*                THE SOFTWARE, EVEN IF MTI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
*                DAMAGES. Because some jurisdictions prohibit the exclusion or
*                limitation of liability for consequential or incidental damages, the
*                above limitation may not apply to you.
*
*                Copyright 2003 Micron Technology, Inc. All rights reserved.
*
****************************************************************************************/

reg timingloaded;

reg [2048 - 1 : 0] TimingName;

typedef enum {
			  TS_AMB ,
			  TS_TM  ,
			  TS_TAP  ,
			  TS_83F  ,
			  TS_83E  ,
			  TS_93F  ,
			  TS_93E  ,
			  TS_93  ,
			  TS_107F ,
			  TS_107E ,
			  TS_107  ,
			  TS_125E  ,
			  TS_125  ,
			  NUM_TS 
			  } UTYPE_TS;

typedef enum {
			  itCK_min ,
			  itCK_max ,
			  itIS ,
			  itIH ,
			  itIPW,
			  itDS ,
			  itDH ,
			  itDIPW,
			  itRC,
			  itCKQK_DLL_DIS,
			  NUM_TIMESPEC    
			  } UTYPE_TIMESPEC;


integer timesets [0:NUM_TS-1][0:NUM_TIMESPEC-1];
reg [0:2048 - 1] timeset_names [0:NUM_TS-1];

UTYPE_TS current_timeset, save_timeset;

task SetTimingDefault;
   begin: SetTimingDefault_task
      SetPatGenTiming(TS_125);
   end
endtask

task SaveTimingSet;
   begin
      save_timeset = current_timeset;
   end
endtask

task RestoreTimingSet;
   begin
      SetPatGenTiming(save_timeset);
   end
endtask

task SettCK;
   input integer itCK;
   begin
      // change tCK here
      timing.tCK = itCK;
      timing.tOffset = itCK ;      
      tCK = timing.tCK;
      tOffset = timing.tCK;

      // change timing that are a function of tCK
   end
endtask

task SetPatGenTiming;
   input UTYPE_TS ts;
   begin: SetPatGenTiming_task
      integer i;

      current_timeset = ts;

      if (timingloaded === 1'bx) LoadTiming;

      $display("Setting PatGen Timing: %0s .....",timeset_names[ts]);
      TimingName = timeset_names[ts];

      SettCK(timesets[ts][itCK_min]); // default
      timing.tOffset = tCK ;      
      
      timing.ClockDutyCycle = 50;	 
      timing.DataClockDutyCycle = 50;	 
 
      timing.tCK_min = timesets[ts][itCK_min];
      timing.tCK_max = timesets[ts][itCK_max];

	  // spec timing
      timing.tIS = timesets[ts][itIS];
      timing.tIH = timesets[ts][itIH];
      timing.tIPW = timesets[ts][itIPW];

      timing.tDS = timesets[ts][itDS];
      timing.tDH = timesets[ts][itDH];
      timing.tDIPW = timesets[ts][itDIPW];
      
      timing.tRC = timesets[ts][itRC];
      
      timing.tCKDK = 0;
      `ifdef DDP
      timing.tCKQK = 125;
      `else
      timing.tCKQK = 100;
      `endif

      timing.tCKQK_DLL_DIS = timesets[ts][itCKQK_DLL_DIS];

      timing.tQKQ = 0;
      timing.tQKVLD = 0;

      timing.tMRSCc = 12;
      timing.tMMDc = 2;
	  
      timing.tDLLkc = 200;
  	  
      tOffset = timing.tOffset ;

      timing.EXP_Advance = 3000;
      
      // call testbench call back function to upload timings
`ifdef RLDRAM3PatGen
      updatetiming();
`endif
   end
endtask

task LoadTiming;
   begin:  LoadTiming_task

      timeset_names[TS_AMB] = "TS_AMB";
      timeset_names[TS_TM] = "TS_TM";
      timeset_names[TS_TAP] = "TS_TAP";
      timeset_names[TS_83F] = "TS_83F";
      timeset_names[TS_83E] = "TS_83E";
      timeset_names[TS_93F] = "TS_93F";
      timeset_names[TS_93E] = "TS_93E";
      timeset_names[TS_93] = "TS_93";
      timeset_names[TS_107F] = "TS_107F";
      timeset_names[TS_107E] = "TS_107E";
      timeset_names[TS_107] = "TS_107";
      timeset_names[TS_125] = "TS_125E";
      timeset_names[TS_125] = "TS_125";
      //                                 TSAMB  TSTM   TSTAP   TS83F  TS83E  TS93F  TS93E   TS93  TS107F TS107E  TS107 TS125E  TS125
      //----------------------------------------------------------------------------------------------------------------------------	 
      SetTSArray   (  itCK_min          , 10000, 10000, 20000,   830,   830,   930,   930,   930,  1070,  1070,  1070,  1250,  1250);
      SetTSArray   (  itCK_max          , 10000, 10000, 20000, 30000, 30000, 30000, 30000, 30000, 30000, 30000, 30000, 30000, 30000);
      SetTSArray   (  itIS              ,  1000,  2500,  1000,   250,   250,   250,   250,   250,   250,   250,   250,   250,   250);
      SetTSArray   (  itIH              ,  1000,  1000,  1000,   250,   250,   250,   250,   250,   250,   250,   250,   250,   250);
      SetTSArray   (  itIPW             ,  1000,  1000,  1000,   470,   470,   470,   470,   470,   535,   535,   535,   560,   560);
      SetTSArray   (  itDS              ,  1000,  1000,  1000,   200,   200,   200,   200,   200,   215,   215,   215,   240,   240);
      SetTSArray   (  itDH              ,  1000,  1000,  1000,    85,    85,    85,    85,    85,   100,   100,   100,   125,   125);
      SetTSArray   (  itDIPW            ,  1000,  1000,  1000,   280,   280,   280,   280,   280,   320,   320,   320,   360,   360);
      SetTSArray   (  itRC              , 12000, 12000, 12000,  6640,  7470,  6510,  7440,  9300,  7490,  7490,  9630,  8000, 12500);
      SetTSArray   (  itCKQK_DLL_DIS    ,  5200,  5200,  5200,  5200,  5200,  5200,  5200,  5200,  5200,  5200,  5200,  5200,  5200);

      timingloaded = 1'b1;

   end
endtask

task SetTSArray;
   input       UTYPE_TIMESPEC index;
   input       integer D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12   ;
   begin
      SetTS(index,TS_AMB  , D0); 
      SetTS(index,TS_TM   , D1); 
      SetTS(index,TS_TAP  , D2); 
      SetTS(index,TS_83F  , D3); 
      SetTS(index,TS_83E  , D4); 
      SetTS(index,TS_93F  , D5); 
      SetTS(index,TS_93E  , D6); 
      SetTS(index,TS_93   , D7); 
      SetTS(index,TS_107F , D8); 
      SetTS(index,TS_107E , D9); 
      SetTS(index,TS_107  , D10); 
      SetTS(index,TS_125E , D11); 
      SetTS(index,TS_125  , D12); 
   end
endtask

function void SetTS;
   input UTYPE_TIMESPEC spec;
   input UTYPE_TS ts;
   input integer value;
   begin
      timesets[ts][spec] = value;
   end
endfunction // SetTS
